/*****************************************************
Chip type           : ATmega8
Program type        : Application
Clock frequency     : 8,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

// CodeVisionAVR C Compiler
// (C) 1998-2004 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATmega8


#pragma used+
sfrb TWBR=0;
sfrb TWSR=1;
sfrb TWAR=2;
sfrb TWDR=3;
sfrb ADCL=4;
sfrb ADCH=5;
sfrw ADCW=4;      // 16 bit access
sfrb ADCSRA=6;
sfrb ADMUX=7;
sfrb ACSR=8;
sfrb UBRRL=9;
sfrb UCSRB=0xa;
sfrb UCSRA=0xb;
sfrb UDR=0xc;
sfrb SPCR=0xd;
sfrb SPSR=0xe;
sfrb SPDR=0xf;
sfrb PIND=0x10;
sfrb DDRD=0x11;
sfrb PORTD=0x12;
sfrb PINC=0x13;
sfrb DDRC=0x14;
sfrb PORTC=0x15;
sfrb PINB=0x16;
sfrb DDRB=0x17;
sfrb PORTB=0x18;
sfrb EECR=0x1c;
sfrb EEDR=0x1d;
sfrb EEARL=0x1e;
sfrb EEARH=0x1f;
sfrw EEAR=0x1e;   // 16 bit access
sfrb UBRRH=0x20;
sfrb UCSRC=0X20;
sfrb WDTCR=0x21;
sfrb ASSR=0x22;
sfrb OCR2=0x23;
sfrb TCNT2=0x24;
sfrb TCCR2=0x25;
sfrb ICR1L=0x26;
sfrb ICR1H=0x27;
sfrw ICR1=0x26;   // 16 bit access
sfrb OCR1BL=0x28;
sfrb OCR1BH=0x29;
sfrw OCR1B=0x28;  // 16 bit access
sfrb OCR1AL=0x2a;
sfrb OCR1AH=0x2b;
sfrw OCR1A=0x2a;  // 16 bit access
sfrb TCNT1L=0x2c;
sfrb TCNT1H=0x2d;
sfrw TCNT1=0x2c;  // 16 bit access
sfrb TCCR1B=0x2e;
sfrb TCCR1A=0x2f;
sfrb SFIOR=0x30;
sfrb OSCCAL=0x31;
sfrb TCNT0=0x32;
sfrb TCCR0=0x33;
sfrb MCUCSR=0x34;
sfrb MCUCR=0x35;
sfrb TWCR=0x36;
sfrb SPMCR=0x37;
sfrb TIFR=0x38;
sfrb TIMSK=0x39;
sfrb GIFR=0x3a;
sfrb GICR=0x3b;
sfrb SPL=0x3d;
sfrb SPH=0x3e;
sfrb SREG=0x3f;
#pragma used-

// Interrupt vectors definitions


// Needed by the power management functions (sleep.h)
#asm
	#ifndef __SLEEP_DEFINED__
	#define __SLEEP_DEFINED__
	.EQU __se_bit=0x80
	.EQU __sm_mask=0x70
	.EQU __sm_powerdown=0x20
	.EQU __sm_powersave=0x30
	.EQU __sm_standby=0x60
	.EQU __sm_ext_standby=0x70
	.EQU __sm_adc_noise_red=0x10
	.SET power_ctrl_reg=mcucr
	#endif
#endasm


    
// I2C Bus functions
#asm
   .equ __i2c_port=0x15 ;PORTC
   .equ __sda_bit=4
   .equ __scl_bit=5
#endasm
/*
  CodeVisionAVR C Compiler
  (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.

  Prototypes for I2C bus master functions

  BEFORE #include -ING THIS FILE YOU
  MUST DECLARE THE I/O ADDRESS OF THE
  DATA REGISTER OF THE PORT AT WHICH
  THE I2C BUS IS CONNECTED AND
  THE DATA BITS USED FOR SDA & SCL

  EXAMPLE FOR PORTB:

    #asm
        .equ __i2c_port=0x18
        .equ __sda_bit=3
        .equ __scl_bit=4
    #endasm
    #include <i2c.h>
*/


#pragma used+
void i2c_init(void);
unsigned char i2c_start(void);
void i2c_stop(void);
unsigned char i2c_read(unsigned char ack);
unsigned char i2c_write(unsigned char data);
#pragma used-


/*
  CodeVisionAVR C Compiler
  (C) 1998-2007 Pavel Haiduc, HP InfoTech S.R.L.

  Prototypes for power management functions
*/


// CodeVisionAVR C Compiler
// (C) 1998-2007 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions


// CodeVisionAVR C Compiler
// (C) 1998-2004 Pavel Haiduc, HP InfoTech S.R.L.

// I/O registers definitions for the ATmega8





											




#pragma used+
#pragma used+
void sleep_enable(void);
void sleep_disable(void);
void idle(void);

void powerdown(void);

void powersave(void);


void standby(void);

void extended_standby(void);
#pragma used-

#pragma library sleep.lib


/*--------------------------------------------------------------------------------------------------

  Name         :  NokiaLCD.h

  Description  :  Header file for Nokia 84x48 graphic LCD driver.

  Author       :  2003-03-08 - Louis Frigon.

  History      :  2003-03-08 - First release.

--------------------------------------------------------------------------------------------------*/


/*--------------------------------------------------------------------------------------------------
                                  General purpose constants
--------------------------------------------------------------------------------------------------*/


//  Mega16 port B pinout for LCD.
//#define LCD_DC_PIN                 0x40  //  PB6
//#define LCD_CE_PIN                 0x10  //  PB4
//#define SPI_MOSI_PIN               0x20  //  PB5
//#define LCD_RST_PIN                0x01  //  PB0
//#define SPI_CLK_PIN                0x80  //  PB7


//  Mega8 port B pinout for LCD.



/*--------------------------------------------------------------------------------------------------
                                       Type definitions
--------------------------------------------------------------------------------------------------*/
typedef char                       bool;
typedef unsigned char              byte;
typedef unsigned int               word;

typedef enum
{
    LCD_CMD  = 0,
    LCD_DATA = 1

} LcdCmdData;

typedef enum
{
    PIXEL_OFF =  0,
    PIXEL_ON  =  1,
    PIXEL_XOR =  2

} LcdPixelMode;

typedef enum
{
    FONT_1X = 1,
    FONT_2X = 2

} LcdFontSize;

/*--------------------------------------------------------------------------------------------------
                                 Public function prototypes
--------------------------------------------------------------------------------------------------*/
void LcdInit       ( void );
void LcdClear      ( void );
void LcdUpdate     ( void );
void LcdClear      ( void );
void LcdGotoXY     ( byte x, byte y );
void LcdChr        ( LcdFontSize size, byte ch );
void LcdStr        ( LcdFontSize size, byte *dataPtr );
void LcdPixel      ( byte x, byte y, LcdPixelMode mode );
void LcdLine       ( byte x1, byte y1, byte x2, byte y2, LcdPixelMode mode );
 void LcdSend ( byte data, LcdCmdData cd );   
void LcdReset(void);
/*--------------------------------------------------------------------------------------------------
                                         End of file.
--------------------------------------------------------------------------------------------------*/


// Standard Input/Output functions
// CodeVisionAVR C Compiler
// (C) 1998-2006 Pavel Haiduc, HP InfoTech S.R.L.

// Prototypes for standard I/O functions


// CodeVisionAVR C Compiler
// (C) 1998-2002 Pavel Haiduc, HP InfoTech S.R.L.

// Variable length argument list macros



typedef char *va_list;





#pragma used+

char getchar(void);
void putchar(char c);
void puts(char *str);
void putsf(char flash *str);

char *gets(char *str,unsigned int len);

void printf(char flash *fmtstr,...);
void sprintf(char *str, char flash *fmtstr,...);
void snprintf(char *str, unsigned int size, char flash *fmtstr,...);
void vprintf (char flash * fmtstr, va_list argptr);
void vsprintf (char *str, char flash * fmtstr, va_list argptr);
void vsnprintf (char *str, unsigned int size, char flash * fmtstr, va_list argptr);
signed char scanf(char flash *fmtstr,...);
signed char sscanf(char *str, char flash *fmtstr,...);




                                   

            


#pragma used-

#pragma library stdio.lib


// CodeVisionAVR C Compiler
// (C) 1998-2000 Pavel Haiduc, HP InfoTech S.R.L.


#pragma used+

void delay_us(unsigned int n);
void delay_ms(unsigned int n);

#pragma used-

// ==============KSH-148 constaint==================
//#define CONST__GAINon  0B10001001 // Weak Signal Booster ON





// ============== LM7001 constaint =================
/*#define CE PORTC.3
#define CL PORTC.2
#define DATA PORTC.1*/

//For LM7001
/*#define CE PORTD.7
#define CL PORTD.6
#define DATA PORTD.5*/

//For Lc72131


//NEW :PD5,PD6,PD7

// ============= Keys constaint ====================

// ============ Modulations ========================

// ============== Global variables =================
char lcd_buffer[33],lcd_buffer_t[33];

char kshset = 0;
char lmset = 0;

unsigned char Mute = 0;

unsigned long WorkFreq = 0L;
//#pragma warn-
eeprom unsigned long WorkFreqEeprom = 106000;
//#pragma warn+

unsigned char NoUpdateScreen = 0;
unsigned long CurrFreq = 101700L;
void SetCompleteFreq(unsigned long Freq);

              
unsigned char adc_input = 0;
unsigned char ADC_Val[2] = {0,0};  //0 for regulator,1 for value
unsigned char agc_buff_pointer = 0;
unsigned char agc_buff_cntr = 0;
unsigned char AGC_Buff[30] = {0};
unsigned int agc_summ = 0;
unsigned char agc_avrg = 0;

unsigned char adc_Iterator = 0;
// ADC interrupt service routine
interrupt [15] void adc_isr(void)
{
        //unsigned char adc_data;
        // Read the 8 most significant bits
        // of the AD conversion result
        //adc_data=ADCH;
        // Place your code here
        #asm("cli")
        
        ADC_Val[adc_input] = ADCH;
        AGC_Buff[agc_buff_pointer] = ADCH;
        ADCSRA |= 0x10; //clear the ADC interrupt flag
        
        agc_summ = 0;
        for(agc_buff_cntr = 0;agc_buff_cntr < 30;agc_buff_cntr++)
             agc_summ += (unsigned int)AGC_Buff[agc_buff_cntr];
        agc_avrg = (unsigned char)(agc_summ / 30);
        
        if(agc_avrg < ADC_Val[0] && Mute == 0)
        {
                Mute = 1;
                NoUpdateScreen = 1;
                SetCompleteFreq(CurrFreq);
                NoUpdateScreen = 0;
        }
        else if(agc_avrg >= ADC_Val[0] && Mute == 1)
        {
                Mute = 0;
                NoUpdateScreen = 1;
                SetCompleteFreq(CurrFreq);
                NoUpdateScreen = 0;
        }
             
        /*if(ADC_Val[1] < ADC_Val[0] && Mute == 0)
        {
                Mute = 1;
                NoUpdateScreen = 1;
                SetCompleteFreq(CurrFreq);
                NoUpdateScreen = 0;
        }
        else if(ADC_Val[1] >= ADC_Val[0] && Mute == 1)
        {
                Mute = 0;
                NoUpdateScreen = 1;
                SetCompleteFreq(CurrFreq);
                NoUpdateScreen = 0;
        }*/
        adc_Iterator++;
        agc_buff_pointer++;
        if(agc_buff_pointer >= 30) agc_buff_pointer = 0;
        
        /*if(adc_input == 0) adc_input = 1;
                else adc_input = 0;*/
        if(adc_Iterator == 0) adc_input = 0;
                else adc_input = 1;        
        #asm("sei")
        ADMUX=adc_input | (0xE0 & 0xff);
        // Start the AD conversion
        if(adc_Iterator == 0 || adc_Iterator == 1) 
               delay_us(10);
        ADCSRA|=0x40;
        
}


void start_conv_adc(unsigned char adc_input)
{
        ADMUX=adc_input | (0xE0 & 0xff);
        //ADMUX=0b1111 | (ADC_VREF_TYPE & 0xff);
        // Delay needed for the stabilization of the ADC input voltage
        //delay_us(10);
        // Start the AD conversion
        ADCSRA|=0x40;
        
}

unsigned char adc_ready(void)
{
        return (ADCSRA & 0x10); 
}
 
unsigned char read_last_adc(void)
{       
        ADCSRA |= 0x10; //clear the ADC interrupt flag
        return ADCH;
}
// Read the 8 most significant bits
// of the AD conversion result
unsigned char read_adc(unsigned char adc_input)
{
        //ADMUX=adc_input | (ADC_VREF_TYPE & 0xff);
        ADMUX=0b1111 | (0xE0 & 0xff);
        // Delay needed for the stabilization of the ADC input voltage
        delay_us(10);
        // Start the AD conversion
        ADCSRA|=0x40;
        // Wait for the AD conversion to complete
        while ((ADCSRA & 0x10)==0);
        ADCSRA|=0x10;
        return ADCH;
}



unsigned char KSH148Read()
{
        unsigned char data;
        
        i2c_start();        
        i2c_write(0b11000001);
        data=i2c_read(0); 
        //printf("KSH_Read ok,status %X\r\n",data);
        i2c_stop();
        return data;        
        

}

unsigned char KSH148Write(unsigned char HI,unsigned char LO)
{
        //unsigned char data;
        i2c_start();        
        i2c_write(0b11000000);
        //printf("KSH_Write %X %X\r\n",HI,LO);
        i2c_write(HI);
        i2c_write(LO);
        i2c_write(0B11001000 );
        i2c_write(0B00000010);
        i2c_stop();
        return 0;        
}
unsigned char HiBYTE(unsigned int value)
{
        return ((value >>8) & 0xFF);
}

unsigned char LoBYTE(unsigned int value)
{
        return (value & 0xFF);
}

//SCL = PB2   PC5
//SDA = PB1   PC4


void delay_(int a)
{
    //delay_us(1000);
    delay_ms(10);

}

void I2CStart(void)
{
     PORTC.4 = 1;
     PORTC.5 = 1;
     //SetBit(I2C_Port,I2C_BitClc);
     //SetBit(I2C_Port,I2C_BitDat);

    delay_(10);
    PORTC.5 = 0;
    // ClrBit(I2C_Port,I2C_BitDat);

    //Pause10(); 
    delay_(10);
    PORTC.4 = 0;
    // ClrBit(I2C_Port,I2C_BitClc);
}

//////////////////
void I2CStop(void)
{    
     PORTC.5 =0;
     PORTC.4 = 1;   
     //ClrBit(I2C_Port,I2C_BitDat);
     //SetBit(I2C_Port,I2C_BitClc);

    //Pause10();
    delay_(10);
    PORTC.5 = 1;
    // SetBit(I2C_Port,I2C_BitDat);
}

//////////////////
void I2CSync(void)
{
    delay_(10);
    //Pause10();
    PORTC.4 = 1;
    // SetBit(I2C_Port,I2C_BitClc);
    delay_(10);
    //Pause10();
    PORTC.4 = 0;
    // ClrBit(I2C_Port,I2C_BitClc);
}
             
void I2CInit(void)
{    
     PORTC.4 = 1;
     PORTC.5 = 1;   
     
     //SetBit(I2C_Port,I2C_BitClc);  // Init I2C
    //SetBit(I2C_Port,I2C_BitDat);

        


}
//////////////////////////
void I2COutByte(unsigned char c)
{
  unsigned char i;
    //printf("O:%d",c);
    for(i=0; i < 8; i++)
    {

      if(c & 0x80)
         PORTC.5 = 1;
	 //SetBit(I2C_Port,I2C_BitDat);
      else
         PORTC.5 = 0;       
	 //ClrBit(I2C_Port,I2C_BitDat);

	I2CSync();

	c <<= 1;
    }
    I2CSync(); // ASK
    return;
}


char SetKSHFrequency(unsigned long FrcvWork)
{
     unsigned char Band;
     unsigned int  fPCH1;
     Band = 0B00000001;
     if(FrcvWork - 38000 > 155000L) Band = 0B00000010;
     if(FrcvWork - 38000 > 440000L) Band = 0B00001100;
     fPCH1 = ((FrcvWork/* + RCVPCH1*/)/50);

     /*I2CStart();
     I2COutByte(KSH148WAdr);   // Tuner Address
     I2COutByte(HiBYTE(fPCH1));
     I2COutByte(LoBYTE(fPCH1));
     I2COutByte(CONST__GAINoff);
     I2COutByte(Band);

     I2CStop();*/  


     //printf("KSH_Write  ADR 0x%X 0x%X 0x%X 0x%X 0x%X\r\n",KSH148WAdr, HiBYTE(fPCH1),LoBYTE(fPCH1),CONST__GAINoff,Band);
     i2c_start();        
     i2c_write(0b11000000);
     i2c_write(HiBYTE(fPCH1));
     i2c_write(LoBYTE(fPCH1));
     i2c_write(0B11001000 ); //CB
     //i2c_write(CONST__GAINon);
     i2c_write(Band);
     i2c_stop();
     delay_ms(1);
     //Reuse associated variables:
     Band = KSH148Read();
     if(Band == 0x7F) return Band;
     for(fPCH1 = 0;fPCH1 < 10;fPCH1++)
     {
          Band = KSH148Read();
          if(Band == 0x7F) return Band;
          delay_ms(10);         
     }
     return Band;   
     
}
// ===================                 ===================




// Declare your global variables here

//SCL = PB2
//SDA = PB1
char ReadData(void)
{
        unsigned char data = 0x13;
        //printf("Start reading\r\n");
        delay_(50);
        
        i2c_start(); 
        //printf("posle start()\r\n");
        i2c_write(0b11000001);      
        //printf("posle write()\r\n");
        i2c_start();        
        //printf("Do i2c_read()\r\n");
        data=i2c_read(0);
        i2c_stop();
        //printf("received %d\r\n",data);



}
void WriteToKSH(char adr)
{               
     //printf("START_W\r\n");   
     I2CStart();
     I2COutByte(adr);   // Tuner Address
     I2COutByte(0b00000111);
     I2COutByte(0b11111111);
     I2COutByte(0B10001001);
     I2COutByte(0B00000010);

     I2CStop();  
     //printf("STOP_W\r\n");





}
                  




// =================== LM7001 routimes ===================
void OutByte(char c)
{
     int i;  

     for(i=0; i < 8; i++)
     {
      if(c & 0x01)
	PORTD.6 = 1;
      else
	PORTD.6 = 0;

      delay_us(5);//delay_us(50); //was 1 ms
      PORTD.5 = 1;
      delay_us(5); //was 1 ms
      PORTD.5 = 0;
      c >>= 1;
    }
    return;
}

void OutToLM(char reg1,char reg2,char reg3)
{
        PORTD.7 = 1;//enable chip
        delay_us(2);//delay_us(50); //was 1 ms
        OutByte(reg1);
        OutByte(reg2);
        OutByte(reg3);
        delay_us(2); //was ms 1
        PORTD.7 = 0;
}


void BusPause(void) { return; }

///////////////////
void BusStrob(void)
{
     BusPause();
	  PORTD.5 = 1;
     BusPause();
	  PORTD.5 = 0;
}

///////////////////////////
void BusOutH2L(unsigned char Bus)
{
  unsigned char i;

    for(i=0; i < 8; i++)
    {
      if(Bus & 0x80)
	 PORTD.6 = 1;
      else
	 PORTD.6 = 0;

      BusStrob();

      Bus <<= 1;
    }
    return;
}

///////////////////////////
void BusOutL2H(unsigned char Bus)
{
  unsigned char i;

    for(i=0; i < 8; i++)
    {
      if(Bus & 0x01)
	 PORTD.6 = 1;
      else
	 PORTD.6 = 0;

      BusStrob();

      Bus >>= 1;
    }
    return;
}



/*#define LC72131_IN1 0b00101000 // 00010100 
#define LC72131_IN2 0b00101001 //10010100 */


void Lc72131Set(unsigned long FreqKHz,unsigned int StepKHz,unsigned char Mode,unsigned char Modulation,unsigned char MuteMode)
{
     unsigned int Devider ;
     unsigned char HByte;
     unsigned char LByte;
     unsigned char StatusReg ;

     Devider = (FreqKHz - 5L) / StepKHz;   
     if(Devider > 0xFFFF)
     {
                //printf("Devider too long : 0x%X",Devider);
                return;
     }

     LByte = LoBYTE(Devider);
     HByte = HiBYTE(Devider);

     
     BusOutH2L(0x14 );
     PORTD.7 = 1;//enable chip
     delay_us(1);//delay_us(50); //was 1 ms
     BusOutL2H(LByte);
     BusOutL2H(HByte);
     BusOutH2L(0x9D);  //step 1 kHz
     delay_us(1); //was ms 1
     PORTD.7 = 0;
	  ///////////////////////////////////////
	  //
	  //  -- Lc72131 IN1 --
	  //
	  ///////////////////////////////////////
	  //
	  //  SNS DVS CTE XS R0 R1 R2 R3
	  //
	  //  R0=0, R1=0, R2=1, R3=0  --> 12,5kHz
	  //  R0=0, R1=1, R2=0, R3=1  -->  5  kHz
	  //  R0=1, R1=1, R2=0, R3=1  -->  1  kHz
	  //
	  //  SNS=1,DVS=0 --> AMIN
	  //
	  //  XS=1 --> 7,2 mHz
	  //
	  //  CTE=1 --> IF counter start
	  //  CTE=0 --> IF counter reset
	  //
	  //  10010010 0x92 -> 12,5 kHz
	  //  10010101 0x95 ->  5,0 kHz
	  //  10011101 0x9D ->  1,0 kHz



     

	  ///////////////////////////////////////
	  //
	  //  -- Lc72131 IN2 --
	  //
	  ///////////////////////////////////////
	  //  IOC1 IOC2 IO1 IO2 BO1 BO2 BO3 BO4
	  //
	  //  BO1 BO2 BO3 BO4 = 0 -> open
	  //  BO1 BO2 BO3 BO4 = 1 -> low
	  //
	  //  IOC1 IOC2 = 0 -> input  mode IO1 IO2
	  //  IOC1 IOC2 = 1 -> output mode IO1 IO2
	  //
	  //  11000001 --> 0xC1
	  //
	  ////////////////////////////////////////
	  //  DNC DOC0 DOC1 DOC2 UL0 UL1 DZ0 DZ1
	  //
	  //  0   0    0    0    0   0   0    0
	  //
	  ////////////////////////////////////////
	  //  GT0 GT1 TBC DLC IFS TEST0 TEST1 TEST2
	  //
	  //  0   0   0   x   1   0     0     0

     
     StatusReg = 0xC0;
     switch(Modulation)
     {                            //(0=AM/1=WFM) (0=MC3362/1=CXA1691) (1=MUTE)
                                  //      
                case 1: StatusReg += 0b011; break;//NFM              
                case 2: StatusReg += 0b100; break;//WFM              
                case 3: StatusReg += 0b101; break;// AM
                
                /*case MOD_NFM: StatusReg += 0b001; break;//NFM              
                case MOD_WFM: StatusReg += 0b110; break;//WFM              
                case MOD_AM: StatusReg += 0b010; break;// AM*/
        
     }

     //if(MuteMode > 0) StatusReg &= 0b000;
     if(MuteMode > 0) StatusReg &= 0b001;
     
     BusOutH2L(0x94 );
     PORTD.7 = 1;//enable chip
     delay_us(1);//delay_us(50); //was 1 ms
     BusOutH2L(StatusReg);
     BusOutH2L(0x00);
     BusOutH2L(0x08);
     delay_us(1); //was ms 1
     PORTD.7 = 0;

     

     return;
}


unsigned char SetLMFreq(unsigned long FreqKHz,unsigned int StepKHz,unsigned char Mode,unsigned char Modulation,unsigned char MuteMode)
{
        unsigned int Devider ;
        unsigned char StatusReg ;
        unsigned char HByte;
        unsigned char LByte;
        unsigned char  Mod4Lm7001[3] = {0xE6,0xE4,0xE2};
        Devider = FreqKHz / StepKHz;
        switch(StepKHz)
        {                            //R 210
                case 100: StatusReg = 0b00000000;break;
                case 50:  StatusReg = 0b01000000;break;
                case 25:  StatusReg = 0b00100000;break;
                case 5:   StatusReg = 0b01100000;break;
                case 10:  StatusReg = 0b00010000;break;
                case 9:   StatusReg = 0b01010000;break;
                case 1:   StatusReg = 0b00110000;break;
        }
        //printf("\r\nDebug after case :Statusbyte [0x%X],Devider =[0x%X]\r\n",StatusReg,Devider);
        if(Mode == 1)
        {
                StatusReg += 0x80; //FM Mode
        }
        else 
        {
                Devider *= 0x10;      //AM Mode Shifting
        }
        if(Devider > 0x3FFF)
        {
                //printf("Devider too long : 0x%X",Devider);
                return 1;
        }
        //StatusReg += 0x00; //NFM              
        //StatusReg += 0x06; //WFM              
        //StatusReg += 0x04; //AM              
        switch(Modulation)
        {                            //(0=AM/1=WFM) (0=MC3362/1=CXA1691) (1=MUTE)
                case 1: StatusReg += 0b001; break;//NFM              
                case 2: StatusReg += 0b110; break;//WFM              
                case 3: StatusReg += 0b010; break;// AM
        
        }
        if(MuteMode > 0) StatusReg |= 0b11;
        LByte = LoBYTE(Devider);
        HByte = HiBYTE(Devider);
        //printf("\r\nDebug : Lbyte [0x%X],Hbyte [0x%X],Statusbyte [0x%X]\r\n",LByte,HByte,StatusReg);
        OutToLM(LByte,HByte,StatusReg);        
        return 0;

}

            
int step = 5;
char Modulation = 2;
unsigned long OldFreq= 0;

unsigned char NoChangeModulation = 0;


unsigned long LMFreq;
unsigned long KSHFreq;
unsigned long OldKSHFreq;

void SetCompleteFreq(unsigned long Freq)
{
        
        unsigned char retcode = 0;
        //unsigned char Modulation = 1;
        
        OldFreq = WorkFreq;
        WorkFreq = Freq;
        
        
        //was(for Fget = 10700 + 38000
        /*if(Freq != OldFreq)
        {
                LMFreq = (RCVPCH1+RCVPCH2)+((WorkFreq % 50) == 0 ? 0 : 50-(WorkFreq % 50));
                KSHFreq = WorkFreq + LMFreq - RCVPCH2;                            
                OldKSHFreq = OldFreq + LMFreq - RCVPCH2;  
        }*/
        if(Freq != OldFreq)
        {
                LMFreq = (38000-10700)-(WorkFreq-((WorkFreq/50)*50));
                KSHFreq = ((WorkFreq + 38000)/50)*50;                            
                OldKSHFreq = ((OldFreq + 38000)%50)*50;
        }
        
        
        //printf("WrkFreq = %ld KSHFreq = %ld LMFreq = %ld",WorkFreq,KSHFreq,LMFreq);
        
        //     :     KSH -   ;      -  
        if(NoChangeModulation == 0)
        {
                if(WorkFreq > 60000L && WorkFreq < 108000L && (OldFreq <= 60000L || OldFreq >= 108000L)) Modulation = 2; //WFM
                else if(WorkFreq > 118000L && WorkFreq < 136000L && (OldFreq <= 118000L || OldFreq >= 136000L)) Modulation = 3; //AM
                        else if((WorkFreq <= 60000L || WorkFreq >= 136000L) && ((OldFreq > 60000L && OldFreq < 108000L) || (OldFreq > 118000L && OldFreq < 136000L))) 
                               Modulation = 1; //NFM
        }
        else NoChangeModulation = 0;
        
        //       
        
        /*if(Mute == 0) retcode = SetLMFreq(LMFreq,5,1,Modulation,1);
        retcode = SetLMFreq(LMFreq,5,1,Modulation,Mute);*/
        
        //if(Mute == 0) Lc72131Set(LMFreq,1,1,Modulation,1);
        Lc72131Set(LMFreq,1,1,Modulation,Mute);

        /*if(retcode != 0)
        {
                //printf("SetLMFreq error! Retcode 0x%X\r\n",retcode);
                lmset = 0;
                //return;
        }*/
        //else lmset = 1;
        /*i2c_init();
        delay_ms(100);     */
        if(KSHFreq != OldKSHFreq) //     
        {
                retcode = SetKSHFrequency(KSHFreq);
                if(retcode != 0x7F)
                {
                        //printf("SetKSHFrequency error! retcode 0x%X\r\n",retcode);
                        
                        kshset = 0;
                        //return;     
                }
                else kshset = 1;
        }
        if(NoUpdateScreen != 1)
        {
                sprintf(lcd_buffer,"%3ld.%03ld",WorkFreq / 1000L, WorkFreq % 1000L);
                LcdClear();
                LcdGotoXY(1,3);  
                LcdStr(FONT_2X,lcd_buffer);
                //LcdUpdate();
                if(step % 5L == 0L || step == 1L) //step also as mode
                {
                        switch(Modulation)
                        {
                                case 1: sprintf(lcd_buffer,"NFM %d KHz",step); break;//NFM              
                                case 2: sprintf(lcd_buffer,"WFM %d KHz",step); break;//WFM              
                                case 3: sprintf(lcd_buffer,"AM  %d KHz",step); break;//AM              
                        
                        }
                }
                if(step == 10001) //modulation changes
                {
                        switch(Modulation)
                        {
                                case 1: sprintf(lcd_buffer,"NFM Mode"); break;//NFM              
                                case 2: sprintf(lcd_buffer,"WFM Mode"); break;//WFM              
                                case 3:  sprintf(lcd_buffer,"AM  Mode"); break;//AM              
                        
                        }
                
                
                }
                if(step == 10002) //iss mode
                {
                        sprintf(lcd_buffer,"ISS Mode"); 
                }                
                if(step == 10003) //scan mode
                {
                        sprintf(lcd_buffer,"SCAN Mode"); 
                }

                
                LcdGotoXY(1,6);  
                LcdStr(FONT_1X,lcd_buffer);
                
                if(kshset != 1)
                {
                        sprintf(lcd_buffer,"KSH Error");
                        LcdGotoXY(5,1);
                        LcdStr(FONT_1X,lcd_buffer);
                }
                LcdUpdate();
                
                LcdReset();
        }
        //ShowFreq(0);
        
}

void EncoderFunc()
{
        //    4  5  = PORTD.2, PORTD.3
        char pin2,tpin2;
        char pin3,tpin3;
        char pin4,tpin4;
        char statemodes[4] = {0,0,0,0};
        char counter = 0,freq_updated = 0;
        
        int mod = 0;
        char scan_mode = 0;
        
        unsigned int cntr = 0;
        unsigned int realstep = step;
        unsigned char y = (48 / 2) + 2;
        unsigned char adc_input_l = 0;
        
        int x =0;
        DDRD.2 = 0;
        DDRD.3 = 0; 
        DDRD.4 = 0;
        PORTD.2 = 1;
        PORTD.3 = 1;
        PORTD.4 = 1;
        pin2 = PIND.2;
        pin3 = PIND.3;
        pin4 = PIND.4;
        //read_adc(0);

        start_conv_adc(0);

        while(1)
        {       
                c_beg:
                tpin2 = PIND.2;
                tpin3 = PIND.3;
                tpin4 = PIND.4;
                cntr++;
                if(cntr > 1000)
                {
                        #asm("sei")
                        start_conv_adc(0);
                }
                else #asm("cli")
                if(cntr >= 500 && cntr % 500 == 0)
                {
                        
                        
                        LcdLine(0,y + adc_input_l,84,y + adc_input_l,PIXEL_OFF);
                        LcdLine(0,y + adc_input_l,(ADC_Val[adc_input_l] / (0xFF / 84)),y + adc_input_l,PIXEL_ON);
                        
                        LcdUpdate();            
                        if(adc_input_l == 0) adc_input_l = 1;
                                else adc_input_l = 0;
                        
                        
                                
                
                }
                for(x =0;x< 10;x++)
                        if(tpin2 != PIND.2 || tpin3 != PIND.3 || tpin4 != PIND.4) //     , ...
                        {       
                                cntr = 0;
                                goto c_beg;
                        }
                if(tpin2 != pin2 || tpin3 != pin3)
                {      
                       cntr = 0;
                       
                       if(counter == 0)
                         if( !((tpin2 == 1 && tpin3 == 0) || (tpin2 == 0 && tpin3 == 1)))
                            continue;
                       if(counter == 1)
                         if( !(tpin2 == 0 && tpin3 == 0))
                            continue;            
                       if(counter == 2)
                         if( !((tpin2 == 1 && tpin3 == 0) || (tpin2 == 0 && tpin3 == 1)))
                            continue; 
                       if(counter == 3)
                         if( !(tpin2 == 1 && tpin3 == 1))
                            continue;                     
                       pin2 = tpin2;
                       pin3 = tpin3;
                       statemodes[counter] = pin2 + pin3*10;
                       counter++;
                       
                       if(pin2 == 1 && pin3 == 1)
                       {
                                
                                counter = 0;
                                if(statemodes[0] == 1 &&  statemodes[1] == 0 && statemodes[2] == 10 && statemodes[3] == 11)
                                {
                                        //printf("-\n");
                                        //WorkFreq -= step;
                                        mod = -1;
                                        freq_updated = 1;
                                }
                                if(statemodes[0] == 10 &&  statemodes[1] == 0 && statemodes[2] == 1 && statemodes[3] == 11)
                                {
                                        //printf("+\n");
                                        //WorkFreq += step;
                                        mod = 1;
                                        freq_updated = 1;
                                }
                                if(freq_updated == 1 && step != 10001 && step != 10002 && step != 10003 && tpin4 == pin4) //   
                                {       
                                        //WorkFreq = mod * step;
                                        if(mod == 1)
                                                CurrFreq += step;
                                        if(mod == -1)
                                                CurrFreq -= step;
                                        SetCompleteFreq(CurrFreq);
                                        freq_updated = 0;
                                }
                                if(freq_updated == 1 && step == 10001  && tpin4 == pin4) // Modulation change    
                                {       
                                        //WorkFreq = mod * step;
                                        Modulation = mod + Modulation;
                                        if(Modulation == 4) Modulation = 1;
                                        if(Modulation == 0) Modulation = 3;
                                        SetCompleteFreq(CurrFreq);
                                        freq_updated = 0;
                                }
                                if(freq_updated == 1 && step == 10002  && tpin4 == pin4) // ISS mode
                                {       
                                        //WorkFreq = mod * step;
                                        if(mod == 1)
                                        {
                                                switch(CurrFreq)
                                                {
                                                       case 143625L: CurrFreq = 130165L;break;
                                                       case 130165L: CurrFreq = 145800L;break;
                                                       case 145800L: CurrFreq = 121750L;break;
                                                       case 121750L: CurrFreq = 145825L;break;
                                                       case 145825L: CurrFreq = 143625L;break;
                                                       default:CurrFreq = 145800L;break;
                                                }
                                        }
                                        if(mod == -1)
                                        {
                                                switch(CurrFreq)
                                                {
                                                       case 130165L: CurrFreq = 143625L;break;
                                                       case 145800L: CurrFreq = 130165L;break;
                                                       case 121750L: CurrFreq = 145800L;break;
                                                       case 145825L: CurrFreq = 121750L;break;
                                                       case 143625L: CurrFreq = 145825L;break;
                                                       default:CurrFreq = 145800L;break;
                                                }
                                        }
                                        NoChangeModulation = 1;
                                        Modulation = 1;
                                        SetCompleteFreq(CurrFreq);
                                        freq_updated = 0;
                                }
                                if(freq_updated == 1 && pin4 == 1) //   
                                {
                                        if(mod == 1)
                                        {
                                                switch(step)
                                                {
                                                        case 1: step = 5;realstep = step;break;
                                                        case 5: step = 10;realstep = step;break;
                                                        case 10: step = 25;realstep = step;break;
                                                        case 25: step = 100;realstep = step;break;
                                                        case 100: step = 1000;realstep = step;break;
                                                        case 1000: step = 10000;realstep = step;break;
                                                        case 10000: step = 10001;break;
                                                        case 10001: step = 10002;break;
                                                        case 10002: step = 10003;break;
                                                        case 10003: step = 1;break;
                                                        
                                                }
                                        }        
                                        if(mod == -1)
                                        {
                                                switch(step)
                                                { 
                                                        case 5: step = 1;realstep = step;break;
                                                        case 10: step = 5;realstep = step;break;
                                                        case 25: step = 10;realstep = step;break;
                                                        case 100: step = 25;realstep = step;break;
                                                        case 1000: step = 100;realstep = step;break;
                                                        case 10000: step = 1000;realstep = step;break;
                                                        case 10001: step = 10000;realstep = step;break;
                                                        case 10002: step = 10001;break;
                                                        case 10003: step = 10002;break;
                                                        case 1: step = 10003;break;
                                                }
                                        }
                                        if(step == 10003) scan_mode = 1; else scan_mode = 0;
                                        SetCompleteFreq(CurrFreq);
                                                
                                }
                                
                       }
                       if(counter >= 4)
                                counter = 0;
                } 
                else 
                {
                        //
                        if(scan_mode > 0 && PIND.4 != 0)
                        {
                                CurrFreq += 5L;
                                SetCompleteFreq(CurrFreq);
                                delay_ms(50);
                                #asm("sei")
                                start_conv_adc(0);
                                /*LcdLine(0,y + adc_input_l,LCD_X_RES,y + adc_input_l,PIXEL_OFF);
                                LcdLine(0,y + adc_input_l,(ADC_Val[adc_input_l] / (0xFF / LCD_X_RES)),y + adc_input_l,PIXEL_ON);
                        
                                LcdUpdate();*/
                                if(adc_input_l == 0) adc_input_l = 1;
                                        else adc_input_l = 0;
                        
                        
                        }
                
                
                
                }
                /*if(tpin4 != pin4)
                {       
                        if(tpin4 == 1 && pin4 == 0)
                        {
                                switch(step)
                                {
                                        case 5: step = 10;break;
                                        case 10: step = 25;break;
                                        case 25: step = 100;break;
                                        case 100: step = 1000;break;
                                        case 1000: step = 10000;break;
                                        case 10000: step = 10001;break;
                                        case 10001: step = 5;break;
                                }
                                
                                SetCompleteFreq(CurrFreq);                        
                        }
                        pin4 = tpin4;
                        
                }*/
        }
}
              
char pluscntr =0;
char minuscntr = 0;        
char delayrecineeprom =0,needwritetoeeprom = 0;
/*
interrupt [TIM2_OVF] void timer2_ovf_isr(void)
{


        char NeedUpdate = 0;
//#asm("cli")        
        if(KeyPlus == 0 && KeyMinus == 0)
        {
                minuscntr =0;
                pluscntr = 0;
                delayrecineeprom++;
                if((delayrecineeprom > delay_wrtoeeprom) && (needwritetoeeprom != 0) && (WorkFreqEeprom != WorkFreq))
                {
                        WorkFreqEeprom = WorkFreq;        
                        needwritetoeeprom = 0;
                        SetCompleteFreq();
                }
        }
        else needwritetoeeprom = 1;
        
                
        if(KeyPlus == 1)
        {    
                
                minuscntr =0;
                if(pluscntr == 0)
                {
                        WorkFreq += 10;
                        NeedUpdate = 1;
                }
                if(pluscntr > delaycntr100)
                {
                        WorkFreq += 100;
                        NeedUpdate = 1;
                }
                if(pluscntr > delaycntr1000)
                {
                        WorkFreq += 1000;
                        NeedUpdate = 1;
                }
                pluscntr++;
                
             
        }
        if(KeyMinus == 1)
        {    
                
                pluscntr =0;
                if(minuscntr == 0) 
                {
                        WorkFreq -= 10;
                        NeedUpdate = 1;
                }
                if(minuscntr > delaycntr100) 
                {
                        WorkFreq -= 100;
                        NeedUpdate = 1;
                }
                if(minuscntr > delaycntr1000) 
                {
                        WorkFreq -= 1000;
                        NeedUpdate = 1;
                }
                minuscntr++;
             
        }
        if(pluscntr > 250) pluscntr = 250;
        if(minuscntr > 250) minuscntr = 250;
        
        if(NeedUpdate >0)
        {    
             WorkFreq = WorkFreq - (WorkFreq % 10);   
             if(pluscntr > delaycntr100 || minuscntr > delaycntr100) 
                ShowFreq(1);
             else ShowFreq(0);
             //SetCompleteFreq();
             NeedUpdate = 0;
             //delay_ms(100);
        }
//#asm("sei")
             
}
  */
// Declare your global variables here
                 
void Shutdown()
{
        sleep_enable();
        powerdown();

}

void main(void)
{     

unsigned long freq = 40000L;
 

#asm("cli")
// Declare your local variables here
//delay_ms(500);

PORTB=0x00;
DDRB=0x3D;
PORTC=0x00;
DDRC=0x00;
PORTD=0x00;
DDRD=0xFF;

 
LcdInit();
  





// Input/Output Ports initialization
// Port B initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTB=0x00;
//DDRB=0x00;

// Port C initialization
// Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTC=0x00;
DDRC=0x00;

// Port D initialization
// Func7=In Func6=In Func5=In Func4=In Func3=In Func2=In Func1=In Func0=In 
// State7=T State6=T State5=T State4=T State3=T State2=T State1=T State0=T 
PORTD=0x00;
DDRD=0x00;



// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: Timer 1 Stopped
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: Off
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x00;
TCNT1H=0x00;
TCNT1L=0x00;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2 output: Disconnected
ASSR=0x00;
TCCR2=0x00;
TCNT2=0x00;
OCR2=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
MCUCR=0x00;

// Timer(s)/Counter(s) Interrupt(s) initialization
TIMSK=0x00;

// // USART initialization
// // Communication Parameters: 8 Data, 1 Stop, No Parity
// // USART Receiver: On
// // USART Transmitter: On
// // USART Mode: Asynchronous
// // USART Baud Rate: 250k
// UCSRA=0x00;
// UCSRB=0x18;
// UCSRC=0x86;
// UBRRH=0x00;
// UBRRL=0x03;

// USART initialization
// Communication Parameters: 8 Data, 1 Stop, No Parity
// USART Receiver: On
// USART Transmitter: On
// USART Mode: Asynchronous
// USART Baud Rate: 56000
UCSRA=0x00;
UCSRB=0x18;
UCSRC=0x86;
UBRRH=0x00;
UBRRL=0x08;

// Analog Comparator initialization
// Analog Comparator: Off
// Analog Comparator Input Capture by Timer/Counter 1: Off
ACSR=0x80;
//SFIOR=0x04;
SFIOR=0x00;

// ADC initialization
// ADC Clock frequency: 1000,000 kHz
// ADC Voltage Reference: AREF pin
// Only the 8 most significant bits of
// the AD conversion result are used
ADMUX=0xE0 & 0xff;
//ADCSRA=0x83;
//WAS: ADCSRA=0b10000011;
ADCSRA=0b10001111;
#asm("sei")
         

// I2C Bus initialization
i2c_init();
//printf("I2C Initialization: Succ\r\n");

  

/*// LCD module initialization
lcd_init(16);
*/


DDRC=0xFC;
DDRD.2 = 1;
DDRD.3 = 1;
PORTD.2 = 0x00;
PORTD.3 = 0x00;

//For LM7001
DDRD.5 = 1;
DDRD.6 = 1;
DDRD.7 = 1;

//DDRD=0x00;
if(WorkFreqEeprom < 0 || WorkFreqEeprom > 1000000) WorkFreqEeprom = 106600;
WorkFreq = WorkFreqEeprom;         


//WorkFreq = 101700L;
SetCompleteFreq(101700L);
SetCompleteFreq(101700L);
SetCompleteFreq(101700L);

EncoderFunc();


/*
while(1)

{       //printf("\r\nWaiting for frequency:");
        scanf("%ld",&freq);
        if(freq == 123456)
                Shutdown();
        //printf("Set freq [%ld]KHz 0x%X\r\n",freq, SetKSHFrequency(freq)); 
        //printf("[%ld %d\r\n",freq, SetLMFreq(freq,5,1)); 
        WorkFreq = freq;
        SetCompleteFreq(WorkFreq);
}
//SetKSHFrequency(128000);
//SetKSHFrequency(128000);
//SetKSHFrequency(128000);

//ShowFreq(1);
          
while (1)
      {
        // Place your code here
        // printf("\r\nWaiting for frequency:");
        scanf("%ld",&freq);
        if(freq == 800000)
                Shutdown();
        //printf("Set freq [%ld]KHz %d\r\n",freq, SetKSHFrequency(freq)); 
        printf("[%ld %d\r\n",freq, SetKSHFrequency(freq)); 
        //ReadData();
        //freq+=1000L;
      };
  */    
} 
